@@ -9,7 +9,7 @@
    zlib.h
 
    The modifications are copyright (C) 2001 Miklos Szeredi
-   (mszeredi@inf.bme.hu)
+   <miklos@szeredi.hu>
 
    The modified software can be distributed under the same licence as
    the original software (see bellow).
